#!/usr/bin/perl -w  # -*- cperl -*- #
#
#  tagcache.pm - A simple object to interface with our tag cache
#
#  GNU MP3D - A portable(ish) MP3 server.
#
# Homepage:
#   http://www.gnump3d.org/
#
# Author:
#  Steve Kemp <steve@steve.org.uk>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
#  Steve Kemp
#  ---
#  http://www.steve.org.uk/


package gnump3d::tagcache;  # must live in tagcache.pm


# set the version for version checking
my $VERSION = '$Revision: 1.6 $';

#
# All the lines read from the tag cache object.
#
my $CACHE_FILE    = "";
my $FORMAT_STRING = "";
my $HIDE_TAGS     = 0;
my @LINES         = ( );


#
#  Create a new instance of this object.
#
sub new
{
    my $class         = shift;
    my $self  = { };
    bless($self, $class);
    return $self;
}


#
#  Set the format string this object will use when formatting the
# audio files.
#
sub setFormatString( )
{
    my $self       = shift;
    $FORMAT_STRING = shift;
}


#
#  Retrieve the format string currently in use.
#
sub getFormatString
{
    my $self = shift;
    return( $FORMAT_STRING );
}


#
#  If called with a positive argume the tag details will be ignored,
# and only filenames will be displayed.
#
sub setHideTags( )
{
    my $self = shift;
    $HIDE_TAGS = shift;
}


#
#  Determine if we're hiding song tags.
#
sub getHideTags( )
{
    my $self = shift;
    return( $HIDE_TAGS );
}


#
#  Return the name of the cache file we're reading from.
#
sub getCacheFile
{
    my $self = shift;
    return( $CACHE_FILE );
}


#
#  Specify the name of the cache file to read from, if one has previously
# been read it's contents will be discarded.
#
sub setCacheFile
{
    my $self = shift;
    my $file = shift;

    if ( ( defined( $CACHE_FILE ) ) and
	 ( $CACHE_FILE ne $file ) )
    {
	@LINES = undef;
	@LINES = ( );
    }
    $CACHE_FILE = $file;
}


#
#   Obtain and format the song tags for a collection of files, this is
# massively faster than doing the same operation on a single file.
#
sub formatMultipleSongTags
{
    my ( $self, @files ) = ( @_ );

    #
    # We return a hash of results - each key is the name of a file,
    # and each value is the formatted result.
    #
    my %RESULTS;

    #
    # Read in the cache file if necessary.
    #
    if ( ( undef( @LINES ) or
	 $#LINES < 0 ) )
    {
	open( FILY, "<$CACHE_FILE" )
	    or die "Cannot read cache file $CACHE_FILE - $!";
	@LINES = <FILY>;
	close( FILY );
    }

    #
    #  Now find the tags for each file, and format them
    #
    foreach my $file ( @files )
    {
	#  Remove double slashes.
	while( $file =~ /\/\// )
	{
	    $file =~ s/\/\//\//g;
	}

	# The formatted tags.
	my $formatted = "";

	print "*** - $file";
	if ( $HIDE_TAGS )
	{
	    print " [hide] ";

	    # Just store the filename.
	    if ( $file =~ /(.*)\/(.*)/ )
	    {
      	        $file = $2;
            }
            if ( $file =~ /(.*)\.(.*)/ )
            {
    	        $file = $1;
            }
	    $formatted = $file;
	}
	else
	{
	    #
	    # Find and format the tags, if present.
	    #
	    $formatted = _formatSingleFile( $file );
	}

	$RESULTS{ $file } = $formatted;
    }

    return( %RESULTS );
}


#
#  Read the tags and format them for the given single file.
#
sub _formatSingleFile ( )
{
    my $file = shift;

    # Holder for the tag values.
    my %TAGS;

    # Did we find any valid tags?
    my $found = 0;

    foreach my $line ( @LINES )
    {
        # Early termination.
        next if ( $found );

	# Strip the newline.
	chomp( $line );

	#
	# If the line starts with the file we are interested in.
	#
	if ( ( $line =~ /^([^\t]+)\t(.*)/ ) and
	     ( $1 eq $file ) )
	{
	    print " +++\n";
	    my $data = $2;

	    my @NAMES = split( /\t/, $data );
	    foreach my $pair ( @NAMES )
	    {
		if ( ( $pair =~ /([A-Z]+)=(.*)/ ) &&
		     ( length( $2 ) ) )
		{
		    $TAGS{ $1 } = $2;
		}
	    }

	    # Convert the file size to something more
	    # readable
	    if ( defined( $TAGS{ 'SIZE' } ) )
	    {
		my $sizeTotal = $TAGS{'SIZE'};

		$sizeTotal = $sizeTotal < (1024)      ?
		    $sizeTotal . " bytes" : (
                      $sizeTotal < (1024 ** 2) ? 
                      (int (10 * $sizeTotal/1024)/10) . "K" : (
                      $sizeTotal < (1024 ** 3) ? 
                      (int (10 * $sizeTotal/(1024 ** 2) )/10) . "Mb" :
                      ((int (10 * $sizeTotal/(1024 ** 3) )/10) . "Gb")));

		$TAGS{'SIZE'} = $sizeTotal;
	    }
	    $found = 1;
	}
    }

    #
    # Not found.
    #
    if ( ! $found )
    {
	print " ---\n";

        # Just store the filename.
        if ( $file =~ /(.*)\/(.*)/ )
        {
      	    $file = $2;
        }
        if ( $file =~ /(.*)\.(.*)/ )
        {
    	    $file = $1;
        }
        $TAGS{ 'FILENAME' } = $file;
    }

    #
    # Work with a copy - we destroy the original
    #
    my $format = $FORMAT_STRING;

    while( $format =~ /(.*)\$([A-Z]+)(.*)/ )
    {
	my $pre  = $1;
	my $key  = $2;
	my $post = $3;

	#
	# Why, oh why did I not use '$TITLE' for the song title?
	#
	if( $key eq "SONGNAME" )
	{
	    $key = 'TITLE';
	}

	if (defined( $TAGS{$key} ) )
	{
	    # Do the insertion.
	    $format = $pre . $TAGS{$key} . $post;
	}
	else
	{
	    $format = $pre . $post;
	}
    }

    #
    #  Make sure we have 'real' tags found.
    #
    my $valid = 0;
    foreach my $key ( keys %TAGS )
    {
	next if ( $key eq 'BITRATE' );
	next if ( $key eq 'CHANNELS' );
	next if ( $key eq 'FILENAME' );
	next if ( $key eq 'FLAG' );
	next if ( $key eq 'LENGTH' );
	next if ( $key eq 'LOWER' );
	next if ( $key eq 'NOMINAL' );
	next if ( $key eq 'RATE' );
	next if ( $key eq 'SIZE' );
	next if ( $key eq 'UPPER' );
	next if ( $key eq 'VERSION' );
	next if ( $key eq 'WINDOW' );
	$valid += 1;
    }


    #
    #  No real tags found - This is necessary as some tags such as file
    # length, and filesize are independent of real tags.
    #
    if ( $valid < 1 )
    {
      $format = $TAGS{'FILENAME'};
    }

    return($format);
}



#
# End of module
#
1;



=head1 NAME

gnump3d::tagcache  - A simple object to interface with our tag cache

=head1 SYNOPSIS


    use gnump3d::tagcache;

    my @files;

    my $tagCache = gnump3d::tagcache->new( );
    $tagCache->setCacheFile( '/tmp/tags.cache' );
    $tagCache->setFormatString( '$ARTIST - SONGNAME' );
    $tagCache->setHideTags( 0 );

    my %TAGS     = $tagCache->formatMultipleSongTags( @files );



=head1 DESCRIPTION

This module implements a simple means of reading and formating the tags
of a group of files en masse from the tag cache created by the gnump3d-index
script


=head2 Methods


=item C<new>

Return a new instance of this object.

=item C<setFormatString>

Set the format string this object will use when formatting the audio
files

=item C<getFormatString>

Get the format string currently in use.

=item C<setHideTags>

Specify whether we should just display the filenames for files, not their tag details.

=item C<getHideTags>

Determine if we're hiding song tags.

=item C<setCacheFile>

Specify the name of the cache file to read when formatting tags.  If a cache file has previously been specified and read its contents will be discarded.

=item C<getCacheFile>

Return the name of the cache file we're going to read the tags from.

=item C<formatMultipleSongTags>

Obtain and format the song tags for a collection of files, this is
massively faster than doing the same operation on a single file.



=back

=head1 AUTHOR

  Part of GNUMP3d, the MP3/OGG/Audio streaming server.

Steve - http://www.gnump3d.org/

=head1 SEE ALSO

L<gnump3d>

=cut
